@@ -17,7 +17,7 @@
 #  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #  MA 02110-1301, USA.
 #
-#  version: 20220827124422
+#  version: 20221215132615
 
 from json import dumps
 import mimetypes
@@ -27,13 +27,13 @@ from io import BytesIO as IO
 from datetime import datetime, timedelta
 from dateutil import tz
 from http import HTTPStatus
-from http.server import BaseHTTPRequestHandler, HTTPServer
+from http.server import BaseHTTPRequestHandler, HTTPServer, ThreadingHTTPServer
 from urllib.parse import urlparse, quote, unquote_plus, parse_qs
 import cgi
 import logging
 
 from .Database import Measurement
-from .Utils import DatetimeEncoder
+from .Utils import DatetimeEncoder, sanitize_braces
 
 
 class InterceptorHandlerFactory:
@@ -46,7 +46,7 @@ class InterceptorHandlerFactory:
     def getHandler(db, static_directory):
         class InterceptorHandler(BaseHTTPRequestHandler):
             querypattern = re.compile(
-                r"^/sensorlog\?hum=(?P<humidity>\d+(\.\d+)?)\&temp=(?P<temperature>\d+(\.\d+)?)\&id=(?P<stationid>[a-z01-9-]+)$",
+                r"^/sensorlog\?hum=(?P<humidity>\d+(\.\d+)?)\&temp=(?P<temperature>-?\d+(\.\d+)?)\&id=(?P<stationid>[a-z01-9-]+)$",
                 re.IGNORECASE,
             )
             htmlpattern = re.compile(
@@ -103,10 +103,10 @@ class InterceptorHandlerFactory:
                 )
 
             def common_headers(self):
-                "the checksum is for the inline script in the updatename.html"
+                """we only allow external scripts from jsdelivr"""
                 self.send_header(
                     "Content-Security-Policy",
-                    "default-src 'self'; script-src 'self' https://cdn.jsdelivr.net/npm/ 'sha256-m/VYsp+J+CFMn+MvBg+XMDwm67RMydRIm5ltf4j4nRk='; object-src 'none'; base-uri 'self'; frame-ancestors 'self';",
+                    "default-src 'self'; script-src 'self' https://cdn.jsdelivr.net/npm/ 'unsafe-inline'; object-src 'none'; base-uri 'self'; frame-ancestors 'self';",
                 )
                 self.send_header("X-Content-Type-Options", "nosniff")
 
@@ -135,43 +135,6 @@ class InterceptorHandlerFactory:
                         )
                         db.storeMeasurement(measurement)
                         self.send_response(HTTPStatus.OK)
-                    elif m := re.match(self.htmlpattern, self.path):
-                        ms = db.retrieveLastMeasurement(m.group("stationid"))
-                        mdivs = "\n".join(
-                            f"""
-                            <div class="measurement{' late' if m['deltat'].total_seconds()>24*3600 else ''}" id="{m["stationid"]}">
-                            <div class="station">{m["name"]}</div>
-                            <div class="time" data-time="{m["time"]}"></div>
-                            <div class="temp">{m["temperature"]:.1f}<span class="degrees">°C</span></div>
-                            <div class="hum">{m["humidity"]:.0f}<span class=percent>%</span></div>
-                            </div>"""
-                            for m in ms
-                        )
-                        html = bytes(
-                            f"""
-<html>
-<head>
-<meta charset="UTF-8">
-<meta http-equiv="refresh" content="300">
-<title>Temperatuur binnen</title>
-<link href="static/css/stylesheet.css" rel="stylesheet"></head>
-<body>
-<div class="measurements">
-{mdivs}
-</div>
-</body>
-<script>document.querySelectorAll("[data-time]").forEach(function(e){{d=new Date(e.getAttribute("data-time"));e.innerHTML=d.getHours() + ":" + d.getMinutes().toString().padStart(2, '0')}})</script>
-</html>
-                        """,
-                            "UTF-8",
-                        )
-                        self.send_response(HTTPStatus.OK)
-                        self.send_header("Content-type", "text/html")
-                        self.send_header("Content-Length", str(len(html)))
-                        self.common_headers()
-                        self.end_headers()
-                        self.wfile.write(html)
-                        return
                     elif m := re.match(self.allpattern, self.path):
                         last_measurements = db.retrieveLastMeasurement()
                         station_data = dumps(
@@ -185,33 +148,22 @@ class InterceptorHandlerFactory:
                             for s in last_measurements
                         }
                         temperature_data_map = dumps(time_series, cls=DatetimeEncoder)
-                        html = bytes(
-                            """<html>
-<script src="https://cdn.jsdelivr.net/npm/jquery@3.6.0/dist/jquery.min.js" integrity="sha256-/xUj+3OJU5yExlq6GSYGSHk7tPXikynS7ogEvDej/m4=" crossorigin="anonymous"></script>
-<script src="https://cdn.jsdelivr.net/npm/chart.js@3.9.1/dist/chart.min.js" integrity="sha256-+8RZJua0aEWg+QVVKg4LEzEEm/8RFez5Tb4JBNiV5xA=" crossorigin="anonymous"></script>
-<script src="https://cdn.jsdelivr.net/npm/chartjs-adapter-date-fns/dist/chartjs-adapter-date-fns.bundle.min.js"></script>
-<head>
-    <meta charset="UTF-8">
-    <meta http-equiv="refresh" content="300">
-    <title>Indoor temperature</title>
-    <link href="static/css/stylesheet.css" rel="stylesheet">
-</head>
-<body>
-    <div id="measurements"></div>
-</body>
-<script>
-    station_data = """
-                            + station_data
-                            + """;
-    temperature_data_map = """
-                            + temperature_data_map
-                            + """;
-</script>
-<script src="static/js/layout.js"></script>
-</html>
-""",
-                            "UTF-8",
-                        )
+
+                        filepath = Path(static_directory) / "all.html"
+                        try:
+                            with open(filepath, "rb") as f:
+                                html = f.read().decode()
+                                html = sanitize_braces(html)
+                                logging.info(html)
+                            html = html.format(
+                                station_data=station_data,
+                                temperature_data_map=temperature_data_map,
+                                timestamp=str(datetime.now()),
+                            )
+                        except FileNotFoundError:
+                            self.send_response(HTTPStatus.NOT_FOUND)
+
+                        html = bytes(html, "UTF-8")
                         self.send_response(HTTPStatus.OK)
                         self.send_header("Content-type", "text/html")
                         self.send_header("Content-Length", str(len(html)))
@@ -272,7 +224,10 @@ class InterceptorHandlerFactory:
                         filepath = Path(static_directory) / (
                             path.decode() if type(path) is bytes else path
                         )  # TODO check encoding URLs are not UTF-8?
-                        InterceptorHandler.checkPath(filepath)
+                        try:
+                            InterceptorHandler.checkPath(filepath)
+                        except ValueError:
+                            self.send_response_only(HTTPStatus.FORBIDDEN)
                         if filepath.is_dir():
                             filepath /= "index.html"
                         mime_type = mimetypes.guess_type(filepath)[0]
@@ -298,7 +253,6 @@ class InterceptorHandlerFactory:
             def do_POST(self):
                 logging.info(self.path)
                 if m := re.match(self.updatenamepattern, self.path):
-                    logging.info(m)
                     file_length = int(self.headers.get("Content-Length", -1))
                     try:
                         keyvalues = parse_qs(
@@ -306,13 +260,14 @@ class InterceptorHandlerFactory:
                         )  # encoding is assumed to be UTF-8
                     except ValueError:
                         self.send_response_only(HTTPStatus.BAD_REQUEST)
-                    logging.info(keyvalues)
+                        self.end_headers()
+                        return
                     stationid = keyvalues.get(b"id", [b""])[0].decode("UTF-8")
                     name = keyvalues.get(b"name", [b""])[0].decode("UTF-8")
-                    logging.info(f"{stationid} {name}")
                     if stationid == "" or name == "":
                         self.send_response_only(HTTPStatus.BAD_REQUEST)
-
+                        self.end_headers()
+                        return
                     names = db.names(stationid, name)
                     # TODO refactor duplicate code
                     names = "\n".join(
@@ -334,7 +289,7 @@ class InterceptorHandlerFactory:
         return InterceptorHandler
 
 
-class Interceptor(HTTPServer):
+class Interceptor(ThreadingHTTPServer):
     allow_reuse_address = True
 
     def __init__(self, server_address, db, static_directory):
